@@ -81,10 +81,10 @@ import (
 // TODO: copy-reduction refactor
 func (mv *Mlrval) ArrayGet(mindex *Mlrval) Mlrval {
 	if !mv.IsArray() {
-		return *ERROR
+		return *FromNotArrayError("array [] base", mv)
 	}
 	if !mindex.IsInt() {
-		return *ERROR
+		return *FromNotIntError("array [] index", mindex)
 	}
 	arrayval := mv.intf.([]*Mlrval)
 	value := arrayGetAliased(&arrayval, int(mindex.intf.(int64)))
@@ -223,12 +223,12 @@ func (mv *Mlrval) ArrayAppend(value *Mlrval) {
 // ================================================================
 func (mv *Mlrval) MapGet(key *Mlrval) Mlrval {
 	if !mv.IsMap() {
-		return *ERROR
+		return *FromNotMapError("map[]", mv)
 	}
 
 	mval, err := mv.intf.(*Mlrmap).GetWithMlrvalIndex(key)
-	if err != nil { // xxx maybe error-return in the API
-		return *ERROR
+	if err != nil {
+		return *FromError(err)
 	}
 	if mval == nil {
 		return *ABSENT
@@ -739,3 +739,51 @@ func LengthenMlrvalArray(array *[]*Mlrval, newLength64 int) {
 		*array = newArray
 	}
 }
+
+// ArrayFold reduces an array to a single value, with a user-supplied starting value and pairwise
+// element-reducer function. Canonical example: start value is 0 and reducer f(a,b) is a+b: this
+// will sum up the values in the array.
+func ArrayFold(
+	a []*Mlrval,
+	initval *Mlrval,
+	f func(a, b *Mlrval) *Mlrval,
+) *Mlrval {
+	acc := initval
+	for _, e := range a {
+		acc = f(acc, e)
+	}
+	return acc
+}
+
+// MapFold reduces a map's values to a single value, with a user-supplied starting value and
+// pairwise element-reducer function. Canonical example: start value is 0 and reducer f(a,b) is a+b:
+// this will sum up the values in the map. Nothing here accesses map keys.
+func MapFold(
+	m *Mlrmap,
+	initval *Mlrval,
+	f func(a, b *Mlrval) *Mlrval,
+) *Mlrval {
+	acc := initval
+	for pe := m.Head; pe != nil; pe = pe.Next {
+		acc = f(acc, pe.Value)
+	}
+	return acc
+}
+
+// CollectionFold multiplexes ArrayFold or MapFold. The panic here is not robust, but is done to
+// avoid adding an error-return that would frictionalize the API.  The idea is that the caller
+// (internal/library functions, not directly user-facing) must have pre-validated that the argument
+// is an array or map. The panic here is merely a fallback, not the primary check.
+func CollectionFold(
+	c *Mlrval,
+	initval *Mlrval,
+	f func(a, b *Mlrval) *Mlrval,
+) *Mlrval {
+	if c.IsArray() {
+		return ArrayFold(c.AcquireArrayValue(), initval, f)
+	} else if c.IsMap() {
+		return MapFold(c.AcquireMapValue(), initval, f)
+	} else {
+		panic("CollectionFold argument is neither array nor map")
+	}
+}
