@@ -33,6 +33,7 @@ var TRANSFORMER_LOOKUP_TABLE = []TransformerSetup{
 	GrepSetup,
 	GroupBySetup,
 	GroupLikeSetup,
+	GsubSetup,
 	HavingFieldsSetup,
 	HeadSetup,
 	HistogramSetup,
@@ -62,9 +63,11 @@ var TRANSFORMER_LOOKUP_TABLE = []TransformerSetup{
 	SortSetup,
 	SortWithinRecordsSetup,
 	SplitSetup,
+	SsubSetup,
 	Stats1Setup,
 	Stats2Setup,
 	StepSetup,
+	SubSetup,
 	SummarySetup,
 	TacSetup,
 	TailSetup,
