@@ -66,7 +66,8 @@ func transformerReshapeUsage(
 	fmt.Fprintf(o, "  Note: if you have multiple regexes, please specify them using multiple -r,\n")
 	fmt.Fprintf(o, "  since regexes can contain commas within them.\n")
 	fmt.Fprintf(o, "  Note: this works with tail -f and produces output records for each input\n")
-	fmt.Fprintf(o, "  record seen.\n")
+	fmt.Fprintf(o, "  record seen.  If input is coming from `tail -f`, be sure to use\n")
+	fmt.Fprintf(o, "  `--records-per-batch 1`.\n")
 	fmt.Fprintf(o, "Long-to-wide options:\n")
 	fmt.Fprintf(o, "  -s {key-field name,value-field name}\n")
 	fmt.Fprintf(o, "  These pivot/reshape the input data to undo the wide-to-long operation.\n")
